@@ -345,6 +345,8 @@ func RunCommand(ctx *ParsingContext, l log.Logger, args []string) (string, error
 	}
 
 	suppressOutput := false
+	disableCache := false
+	useGlobalCache := false
 	currentPath := filepath.Dir(ctx.TerragruntOptions.TerragruntConfigPath)
 	cachePath := currentPath
 
@@ -356,8 +358,21 @@ func RunCommand(ctx *ParsingContext, l log.Logger, args []string) (string, error
 
 			args = slices.Delete(args, 0, 1)
 		case "--terragrunt-global-cache":
+			if disableCache {
+				return "", errors.New(ConflictingRunCmdCacheOptionsError{})
+			}
+
+			useGlobalCache = true
 			cachePath = "_global_"
 
+			args = slices.Delete(args, 0, 1)
+		case "--terragrunt-no-cache":
+			if useGlobalCache {
+				return "", errors.New(ConflictingRunCmdCacheOptionsError{})
+			}
+
+			disableCache = true
+
 			args = slices.Delete(args, 0, 1)
 		default:
 			checkOptions = false
@@ -368,15 +383,18 @@ func RunCommand(ctx *ParsingContext, l log.Logger, args []string) (string, error
 	// see: https://github.com/gruntwork-io/terragrunt/issues/1427
 	cacheKey := fmt.Sprintf("%v-%v", cachePath, args)
 
-	cachedValue, foundInCache := runCommandCache.Get(ctx, cacheKey)
-	if foundInCache {
-		if suppressOutput {
-			l.Debugf("run_cmd, cached output: [REDACTED]")
-		} else {
-			l.Debugf("run_cmd, cached output: [%s]", cachedValue)
-		}
+	// Skip cache lookup if --terragrunt-no-cache is set
+	if !disableCache {
+		cachedValue, foundInCache := runCommandCache.Get(ctx, cacheKey)
+		if foundInCache {
+			if suppressOutput {
+				l.Debugf("run_cmd, cached output: [REDACTED]")
+			} else {
+				l.Debugf("run_cmd, cached output: [%s]", cachedValue)
+			}
 
-		return cachedValue, nil
+			return cachedValue, nil
+		}
 	}
 
 	cmdOutput, err := shell.RunCommandWithOutput(ctx, l, ctx.TerragruntOptions, currentPath, suppressOutput, false, args[0], args[1:]...)
@@ -392,9 +410,11 @@ func RunCommand(ctx *ParsingContext, l log.Logger, args []string) (string, error
 		l.Debugf("run_cmd output: [%s]", value)
 	}
 
-	// Persisting result in cache to avoid future re-evaluation
+	// Persisting result in cache to avoid future re-evaluation, unless --terragrunt-no-cache is set
 	// see: https://github.com/gruntwork-io/terragrunt/issues/1427
-	runCommandCache.Put(ctx, cacheKey, value)
+	if !disableCache {
+		runCommandCache.Put(ctx, cacheKey, value)
+	}
 
 	return value, nil
 }
@@ -711,20 +731,28 @@ func ParseTerragruntConfig(ctx *ParsingContext, l log.Logger, configPath string,
 		return cty.NilVal, err
 	}
 
+	// Preserve the ParserOptions from the parent context when creating the new context
+	// This ensures that error suppression handlers and other parser options are propagated
+	// to nested read_terragrunt_config calls
+	parserOptions := ctx.ParserOptions
+
 	ctx = ctx.WithTerragruntOptions(opts)
+	if len(parserOptions) > 0 {
+		ctx = ctx.WithParseOption(parserOptions)
+	}
 
 	// check if file is stack file, decode as stack file
 	if strings.HasSuffix(targetConfig, DefaultStackFile) {
 		stackSourceDir := filepath.Dir(targetConfig)
 
-		values, err := ReadValues(ctx, l, opts, stackSourceDir)
-		if err != nil {
-			return cty.NilVal, errors.Errorf("failed to read values from directory %s: %v", stackSourceDir, err)
+		values, readErr := ReadValues(ctx, l, opts, stackSourceDir)
+		if readErr != nil {
+			return cty.NilVal, errors.Errorf("failed to read values from directory %s: %v", stackSourceDir, readErr)
 		}
 
-		stackFile, err := ReadStackConfigFile(ctx, l, opts, targetConfig, values)
-		if err != nil {
-			return cty.NilVal, errors.New(err)
+		stackFile, readErr := ReadStackConfigFile(ctx, l, opts, targetConfig, values)
+		if readErr != nil {
+			return cty.NilVal, errors.New(readErr)
 		}
 
 		return stackConfigAsCty(stackFile)
@@ -732,9 +760,9 @@ func ParseTerragruntConfig(ctx *ParsingContext, l log.Logger, configPath string,
 
 	// check if file is a values file, decode as values file
 	if strings.HasSuffix(targetConfig, valuesFile) {
-		unitValues, err := ReadValues(ctx.Context, l, ctx.TerragruntOptions, filepath.Dir(targetConfig))
-		if err != nil {
-			return cty.NilVal, errors.New(err)
+		unitValues, readErr := ReadValues(ctx.Context, l, ctx.TerragruntOptions, filepath.Dir(targetConfig))
+		if readErr != nil {
+			return cty.NilVal, errors.New(readErr)
 		}
 
 		return *unitValues, nil
